﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Globalization;

namespace HabboTraxDEBESTE2
{
    public class TraxSong
    {
        private List<TraxPage> pages;
        private int currentPage;
        private TraxPage pagePlaying;
        private string title;
        public string Title
        {
            get { return title; }
            set { title = value; }
        }
        public EventHandler FinishedPlaying;
        private EventHandler<SlotEventArgs> OnNextPageStarts;
        private EventHandler<SlotEventArgs> OnNextSlotPlays;
        public Action<int> NextPageStarts;
        public Action<int> NextSlotPlays;

        public TraxSong()
        {
            pages = new List<TraxPage>();
            title = string.Empty;
            OnNextSlotPlays += (s, e) =>
                {
                    if (NextSlotPlays != null)
                        NextSlotPlays(e.Slot);
                };
            AddPage();
            currentPage = 0;
        }

        public void AddPage()
        {
            TraxPage newPage = new TraxPage();
            newPage.OnSlotStartedPlaying += OnNextSlotPlays;
            pages.Add(newPage);

        }

        /// <summary>
        /// Increases the page length of the last page in the song
        /// </summary>
        public void IncreasePageLength()
        {
            IncreasePageLength(pages.Count() - 1);
        }

        public void IncreasePageLength(int pageNr)
        {
            pages[pageNr].AddSlots(24);
        }

        public TraxPage GetPage(int i)
        {
            return pages[i];
        }

        public TraxPage SelectNextPage()
        {
            if (currentPage+1 < TotalPages)
                currentPage++;
            return CurrentTraxPage();
        }

        public TraxPage SelectPreviousPage()
        {
            if (currentPage > 0)
                currentPage--;
            return CurrentTraxPage();
        }

        public TraxPage CurrentTraxPage()
        {
            return GetPage(currentPage);
        }

        public int TotalPages
        {
            get { return pages.Count(); }
        }

        public void Play(Player player)
        {
            TraxPage firstPage = pages.FirstOrDefault();
            if (firstPage != null)
            {
                pagePlaying = firstPage;
                firstPage.FinishedPlaying += (s, e) =>
                    {
                        Play(player, s as TraxPage);
                    };
                firstPage.Play(player, pages.Count > 1);
            }
        }

        public void Stop()
        {
            if (pagePlaying != null)
                pagePlaying.Stop();
        }

        public void Play(Player player, TraxPage lastPage)
        {
            TraxPage next = GetNextPage(lastPage);
            if (next != null)
            {
                next.Play(player, pages.IndexOf(next) < pages.Count - 1);
                pagePlaying = next;
                next.FinishedPlaying += (s, e) =>
                    {
                        Play(player, next);
                    };
            }
            else
            {
                pagePlaying = null;
                if (FinishedPlaying != null)
                    FinishedPlaying(this, null);
            }
        }

        public TraxPage GetNextPage(TraxPage page)
        {
            if (page != null)
            {
                int currentPage = pages.IndexOf(page);
                if (currentPage + 1 < TotalPages)
                    return pages[currentPage + 1];
                else
                    return null;
            }
            return null;
        }

        public TraxPage getPage(int index)
        {
            return pages[index];
        }

        public void LoadAllTraxIntoPlayer(Player player)
        {
            foreach (TraxPage page in pages)
                page.LoadTraxToPlayer(player);
        }

        public void SaveToXml(string filename, string[] openedCDs)
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;

            XmlWriter textWriter = XmlWriter.Create(filename, settings);
            try
            {
                textWriter.WriteStartDocument();
                textWriter.WriteComment("This is a Habbo Trax file!");
                textWriter.WriteComment("Made by Bas Dado's Habbo Trax editor");

                //Write the song data
                textWriter.WriteStartElement("Song");
                textWriter.WriteAttributeString("title", Title);
                textWriter.WriteAttributeString("pageCount", pages.Count().ToString(CultureInfo.InvariantCulture));
                textWriter.WriteAttributeString("lastOpenPage", currentPage.ToString(CultureInfo.InvariantCulture));
                textWriter.WriteStartElement("Pages");
                textWriter.WriteAttributeString("totalNr", pages.Count().ToString(CultureInfo.InvariantCulture));
                foreach (TraxPage page in pages)
                {
                    page.WriteToXml(textWriter);
                }
                textWriter.WriteEndElement(); //Close "Pages"

                //Write what CDs the user had opened
                textWriter.WriteStartElement("OpenCDs");
                foreach (string cd in openedCDs)
                    textWriter.WriteElementString("cd", cd);
                textWriter.WriteEndElement(); //Close "OpenCDs"

                textWriter.WriteEndElement(); //Close "Song"



                textWriter.WriteEndDocument();
            }
            catch (Exception e)
            {
                textWriter.Flush();
                textWriter.Close();
                Console.WriteLine(e.ToString());
                throw e;
            }

            textWriter.Flush();
            textWriter.Close();

        }

        public static TraxSong LoadFile(string filename, out Trax[] traxToLoadOut, out string[] cdsToLoad)
        {
            XmlReader reader = XmlReader.Create(filename);
            TraxSong loadedSong = new TraxSong();
            List<Trax> traxToLoad = new List<Trax>();
            List<string> openCDs = new List<string>();
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Song")
                {
                    loadedSong.Title = reader.GetAttribute("title") ?? string.Empty;
                    loadedSong.currentPage = int.Parse(reader.GetAttribute("lastOpenPage"));
                }
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "Pages")
                {
                    loadedSong.pages = new List<TraxPage>();
                    while (reader.Read() && !(reader.NodeType == XmlNodeType.EndElement && reader.Name == "Pages"))
                    {
                        if (reader.NodeType == XmlNodeType.Element && reader.Name == "Page") {
                            Trax[] traxToLoadForPage;
                            TraxPage newPage = TraxPage.LoadFromXml(reader, out traxToLoadForPage);
                            newPage.OnSlotStartedPlaying += loadedSong.OnNextSlotPlays;
                            loadedSong.pages.Add(newPage);
                            foreach (Trax trax in traxToLoadForPage)
                                if (!traxToLoad.Contains(trax))
                                    traxToLoad.Add(trax);
                        }
                    }

                }
                if (reader.NodeType == XmlNodeType.Element && reader.Name == "OpenCDs")
                    while (reader.Read() && !(reader.NodeType == XmlNodeType.EndElement && reader.Name == "OpenCDs"))
                    {
                        if (reader.NodeType == XmlNodeType.Element && reader.Name == "cd") {
                            reader.Read();
                            if (reader.NodeType == XmlNodeType.Text)
                                openCDs.Add(reader.Value);
                        }
                    }
            }
            cdsToLoad = openCDs.ToArray();
            traxToLoadOut = traxToLoad.ToArray();
            reader.Close();
            return loadedSong;
        }
        

    }
}
